{*****************************************************************************
 *
 *  DataMod.pas - Central Data Module
 *
 *  Copyright (c) 1999 Diego Amicabile
 *
 *  Author:     Diego Amicabile
 *  E-mail:     diegoami@yahoo.it
 *  Homepage:   http://www.geocities.com/diegoami
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation;
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *----------------------------------------------------------------------------
 *
 *  Revision history:
 *
 *     DATE     REV                 DESCRIPTION
 *  ----------- --- ----------------------------------------------------------
 *
 *****************************************************************************}

unit DataMod;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBTables, Db, inifiles{, uLogger}, ObjGraph, uSearchRoutine;

type
  TRoutesDataModule = class(TDataModule)
    Database: TDatabase;
    RoutesDataSource: TDataSource;
    RoutesTable: TTable;
    CreateRouteQuery: TQuery;
    RoutesQuery: TQuery;
    TownsQuery: TQuery;
    HourQuery: TQuery;
    RouteSearchQuery: TQuery;
    CreateDatabaseQuery: TQuery;
    OtherWayTable: TTable;
    OtherWayDataSource: TDataSource;
    TownListQuery: TQuery;
    CreateNodesQuery: TQuery;
    CreateEdgesQuery: TQuery;
    GetHourToQuery: TQuery;
    procedure RoutesDataModuleCreate(Sender: TObject);
    procedure CreateTable(TableName : String; Fields : TStrings);
    procedure DropTable(TableName : String);
    procedure RoutesDataModuleDestroy(Sender: TObject);
//    function GetConnections(Town : String; Route : String; Arrival : String; level : smallint) : boolean;
    function GetConnectionList(Departure, Arrival : String; HOur : String; Day : Char) : TEdgeList;

  protected
    RouteGraph : TMathGraph;
    NewEdgeList : TEdgeList;


  public
    DataBaseDir : String;
    URLLocation : String;
    ILFileIni   : TIniFIle;

    procedure CreateDatabase;
    procedure Update(Directory : String);
    procedure CreateRouteStructure;
    function GetHourTo(RouteEdge : TRouteEdge; Hour : String) : String;
  end;

var
  RoutesDataModule: TRoutesDataModule;


//function TownSortOnLevel(Item1, Item2 : Pointer) : integer;

implementation

{$R *.DFM}


function GetTownNode(Town : String; Graph : TMathGraph) : TTownNode;
var i : integer;
  TownNode : TTownNode;
begin
  for i := 1 to Graph.NodeList.Size do begin
    TownNode := TTownNode(Graph.NodeList.At[i]);
    if TownNode.Town.Str = Town then begin
      result := TownNode;
      Exit;
    end;
  end;
  result := nil;
end;

function GetWinDir : String;
var
  I : Integer;
begin
  SetLength(Result,MAX_PATH+1);
  I := GetWindowsDirectory(PChar(Result),MAX_PATH);
  SetLength(Result,I);
end;

procedure TRoutesDataModule.CreateRouteStructure;
var i, j : integer;
  Town : String;
  FR, CR, FT, CT : String;
  TownNode : TTownNode;
  TownString : TStringObj;


begin
  RouteGraph.NodeList.Clear;
  RouteGraph.EdgeList.Clear;
  CreateNodesQuery.Open;
  CreateNodesQuery.First;

  while (not CreateNodesQuery.EOF) do begin
    Town := CreateNodesQuery.Fields[0].AsString;
    TownString := TStringObj.Create(Town);
    TownNode := TTownNode.Create(TownString);
    RouteGraph.NodeList.InsertAtTail(TownNode);
    CreateNodesQuery.Next;
  end;
  CreateEdgesQuery.Open;
  CreateEdgesQuery.First;
  CR := CreateEdgesQuery.Fields[0].AsString;
  CT := CreateEdgesQuery.Fields[1].AsString;
  while (not CreateEdgesQuery.EOF) do begin
    CreateEdgesQuery.Next;
    if not CreateEdgesQuery.EOF then begin
      FR := CR;
      FT := CT;
      CR := CreateEdgesQuery.Fields[0].AsString;
      CT := CreateEdgesQuery.Fields[1].AsString;
      if CR = FR then
        RouteGraph.EdgeList.Add(TRouteEdge.Create(GetTownNode(FT, RouteGraph), GetTownNode(CT, RouteGraph), CR));
    end;
  end;
end;

function TRoutesDataModule.GetHourTo(RouteEdge : TRouteEdge; Hour : String) : String;
var HourTo, FirstHour, FirstVec,FirstDep, Vehicle, HourFrom : string;
  TownTo, TownFrom : String;
  function FieldToString(Field : TTimeField) : String;
  var Time : TDateTime;
      TimeStr : String;
      Hour, Min, Sec, MSec : Word;
      hh, mm : String;
  begin
    Time := Field.AsDateTime;
    DecodeTime(Time, Hour, Min, Sec, MSec);
    hh := IntToStr(Hour);
    mm := IntToStr(Min);
    if Length(hh) = 1 then
      hh := '0'+hh;
    if Length(mm) = 1 then
      mm := '0'+mm;
    result := hh+'.'+mm;
  end;

var
  DayChars : String;
  TownNode, ToTownNode : TTownNode;
  StringToAdd : STring;
begin
  Hour := HourToString(Hour);
  SetLength(DayChars,1);
  with GetHourToQuery do begin
    Close;
    SQL.Clear;
    TownNode := (RouteEdge.FromNode As TTownNode);

    ToTownNode := (RouteEdge.ToNode As TTownNode);
    TownTo := ToTownNode.Town.Str;
    TownFrom := TownNode.Town.Str;
    StringTOAdd := 'SELECT Vehicle, '+TownTo+',  '+TownFrom+' FROM '+RouteEdge.Route;
    if TownNode.Day <> ' ' then begin
      DayChars[1] := TownNode.Day;
      StringToAdd := StringTOAdd +' WHERE DAYS LIKE "%'+DayChars+'%" UNION ' +
         StringToAdd + ' WHERE DAYS = ""';
    end;
    //Log ('Hour = '+Hour+', TownFrom = ' + TownFrom+ ', TownTo = ' +TownTo);
    SQL.Add(StringToAdd);
    Open;
    First;
    FirstVec := Fields[0].AsString;
    FirstDep := FieldToString(Fields[2] As TTimeField);
    FirstHour := FieldToString(Fields[1] As TTimeField);
    while not EOF do begin
      Vehicle := Fields[0].AsString;
      HourFrom := FieldToString(Fields[2] As TTimeField);
      HourTo := FieldToString(Fields[1] As TTimeField);
      if HourFrom >= Hour then break;
      Next;
    end;
    if HourFrom >= Hour then begin
      result := HourTo;
      RouteEdge.DepHour := HourFrom;
      RouteEdge.Vehicle := Vehicle
    end else begin
      result := FirstHour;
      RouteEdge.Vehicle := FirstVec;
      RouteEdge.DepHour  := FirstDep

    end;
  end;
end;

procedure TRoutesDataModule.RoutesDataModuleCreate(Sender: TObject);
var Directory : String;
begin
  IlFileIni := TIniFile.Create(GetWinDir+'\STTM.ini');
  Directory := IlFileIni.ReadString('Main','Database',GetCurrentDir+'\nordest');
  URLLocation := IlFileIni.ReadString('Main','URL','http:\\localhost\scripts\timeserv.exe?database=nordest');
  Update(Directory);
  RouteGraph := TMathGraph.Create;
  NewEdgeList := TEdgeList.Create;
end;


procedure TRoutesDataModule.Update(Directory : String);
begin
  if Pos('\',Directory) = 0 then
    DatabaseDir := GetCurrentDir+'\'+Directory
  else
    DatabaseDir := Directory;
  RoutesTable.DatabaseName         := DatabaseDir;
  TownListQuery.DatabaseName         := DatabaseDir;
  CreateRouteQuery.DatabaseName    := DatabaseDir;
  RoutesQuery.DatabaseName         := DatabaseDir;
  TownsQuery.DatabaseName          := DatabaseDir;
  HourQuery.DatabaseName           := DatabaseDir;
  RouteSearchQuery.DatabaseName    := DatabaseDir;
  CreateDatabaseQuery.DatabaseName := DatabaseDir;
  OtherWayTable.DatabaseName       := DatabaseDir;
  CreateNodesQuery.DatabaseName       := DatabaseDir;
  CreateEdgesQuery.DatabaseName       := DatabaseDir;
  GetHourToQuery.DatabaseName       := DatabaseDir;

end;

function Quote( const SQLString: String ): String;
begin
  if ( SQLString = '' ) then
    Result := ''
  else
    Result := Chr(39) + SQLString + Chr(39);
end;

procedure TRoutesDataModule.DropTable(TableName : String);
begin
  with CreateRouteQuery do begin
    Close;
    SQL.Clear;
    SQL.Add('DROP TABLE '+TableName );
    ExecSQL;
    Close;
    SQL.Clear;
    SQL.Add('DELETE FROM ROUTESTOWNS ');
    SQL.Add('WHERE ROUTE = '+Quote(TableName));
    ExecSQL;
  end;
end;

procedure TRoutesDataModule.CreateDatabase;
begin
  try
    CreateDataBaseQuery.ExecSQL;
  except on EDBEngineError do
    if MessageDlg('Database Exists. Do you want to drop it?',mtConfirmation,[mbYes,mbNo],0) = mrYes then begin
       DropTable('RoutesTowns');
       CreateDatabase;
    end;
  end;
end;

procedure TRoutesDataModule.CreateTable(TableName : String; Fields : TStrings);

  function SpacesToLines(aString : String) : String;
  var i : integer;
  begin
    for i := 1 to length(aString) do
      if aString[i] = ' ' then
        aString[i] :='_';
    result := aString;
  end;

var I : smallint;
begin
  try
    with CreateRouteQuery do begin
      Close;
      SQL.Clear;
      SQL.Add('CREATE TABLE '+TableName );
      SQL.Add('( VEHICLE VARCHAR(20),');
    end;
      for i := 0 to Fields.Count-1 do
        CreateRouteQuery.SQL.Add(SpacesToLines(Fields.Strings[i])+' TIME,');
    with CreateRouteQuery do begin
      SQL.Add('DAYS  VARCHAR(7), ');
      SQL.Add('NOTES  VARCHAR(50) );');
      ExecSQL;
    end;

  except on EDBEngineError do
    if MessageDlg('Table Exists. Do you want to drop it?',mtConfirmation,[mbYes,mbNo],0) = mrYes then begin
      DropTable(TableName);
      CreateTable(TableName,Fields);
    end;
  end;
  with CreateRouteQuery do begin
    Close;
    SQL.Clear;
  end;
  for i := 0 to Fields.Count-1 do begin
    CreateRouteQuery.SQL.Add('INSERT INTO ROUTESTOWNS ');
    CreateRouteQuery.SQL.Add('VALUES ( '+Quote(TableName)+','+
      Quote(SpacesToLines(Fields.Strings[i]))+','+IntToStr(i)+')');
    CreateRouteQuery.ExecSQL;
    CreateRouteQuery.SQL.Clear;
  end;
end;

procedure TRoutesDataModule.RoutesDataModuleDestroy(Sender: TObject);
begin
  IlFileIni.WriteString('Main','Database',DatabaseDir);
  IlFileIni.WriteString('Main','URL',URLLocation);
end;

function TRoutesDataModule.GetConnectionList(Departure, Arrival : String; Hour : String; Day : Char) : TEdgeList;
var DN, AN : TTownNode;
  TotalEdgeList : TEdgeList;
  j : integer;
  VV: String;
  FVV : String;
  Edge, CurrentEdge, FormerEdge : TRouteEdge;

begin
  DN := GetTownNode(Departure,RouteGraph) As TTownNode;
  DN.Hour := Hour;
  DN.Day := Day;
  AN := GetTownNode(Arrival, RouteGraph) As TTownNode;
  TotalEdgeList := RouteGraph.GetShortestPath(DN, AN);
  NewEdgeList.Clear;

  Edge := TRouteEdge(TotalEdgeList.Last);
  VV := Edge.Vehicle;
  CurrentEdge := TRouteEdge.Create(Edge.FromNode,nil,VV);
  CurrentEdge.Vehicle := VV;
  CurrentEdge.DepHour := Edge.DepHour;
  NewEdgeList.Add(CurrentEdge);
  for j := TotalEdgeList.Count-2 downto 0 do begin
    FormerEdge := Edge;
    Edge := TRouteEdge(TotalEdgeList.Items[j]);
    FVV := VV;
    VV := Edge.Vehicle;
    if FVV <> VV then begin
      CurrentEdge.ToNode := FormerEdge.ToNode;
      CurrentEdge := TRouteEdge.Create(Edge.FromNode, nil, VV);
      CurrentEdge.Vehicle := VV;
      CurrentEdge.DepHour := Edge.DepHour;
      NewEdgeList.Add(CurrentEdge);
    end;
  end;
  CurrentEdge.ToNode := Edge.ToNode;
  result := NewEdgeList;
end;

end.
