{*****************************************************************************
 *
 *  Search.pas - Dialog to perform Search
 *
 *  Copyright (c) 1999 Diego Amicabile
 *
 *  Author:     Diego Amicabile
 *  E-mail:     diegoami@yahoo.it
 *  Homepage:   http://www.geocities.com/diegoami
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation;
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *----------------------------------------------------------------------------
 *
 *  Revision history:
 *
 *     DATE     REV                 DESCRIPTION
 *  ----------- --- ----------------------------------------------------------
 *
 *****************************************************************************}

unit Search;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Db, DBTables, HTTPApp, shellapi, Mask, ComCtrls;
type
  TSearchForm = class(TForm)
    RouteQueryPanel: TPanel;
    ResultPanel: TPanel;
    DepartureLabel: TLabel;
    ArrivalLabel: TLabel;
    DepartureTimeLabel: TLabel;
    Combobox: TComboBox;
    SubmitButton: TButton;
    URLEdit: TEdit;
    Label1: TLabel;
    HourEdit: TMaskEdit;
    ListView: TListView;
    DepartureComboBox: TComboBox;
    ArrivalComboBox: TComboBox;
    ProcessingLabel: TLabel;
    BrowserCheckBox: TCheckBox;
    ExitButton: TButton;
    procedure SearchButtonClick(Sender: TObject);
    procedure SubmitButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure ListViewCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure ExitButtonClick(Sender: TObject);



  public
    { Public declarations }
  end;
var
  SearchForm: TSearchForm;


implementation

uses DataMod, objGraph, uSearchRoutine;


{$R *.DFM}
procedure TSearchForm.SearchButtonClick(Sender: TObject);
var Departure, Arrival: String;
   j : Integer;
   ListItem : TListItem;
   EdgeList : TEdgeList;
   Hour : String;
   TF, TT : TTownNode;
   RouteEdge : TRouteEdge;
   Day : Char;
begin

  ListView.Items.Clear;
  Departure := DepartureComboBox.Items[DepartureComboBox.ItemIndex];
  Arrival   := ArrivalComboBox.Items[ArrivalComboBox.ItemIndex];
  Hour := HourEdit.Text;
  if not ValidHour(Hour) then
    raise EConvertError.Create('Invalid Time')
  else if (Length(Arrival) = 0) or (Length(Departure) = 0) then
    raise Exception.Create('Uncomplete data')
  else begin
    Day := DayToChar(ComboBox.Items[ComboBox.ItemIndex]);
    EdgeList := RoutesDataModule.GetConnectionList(Departure, Arrival, Hour, Day);
    ListView.Items.BeginUpdate;
    for j := 0 to EdgeList.Count-1 do begin
      RouteEdge := TRouteEdge(EdgeList.Items[j]);
      TF := TTownNode(RouteEdge.FromNode);
      TT := TTownNode(RouteEdge.ToNode);
      ListItem := ListView.Items.Add;
      ListItem.Caption := RouteEdge.Vehicle;
      ListItem.SubItems.Add(TF.Town.Str);
      ListItem.SubItems.Add(RouteEdge.DepHour);
      ListItem.SubItems.Add(TT.Town.Str);
      ListItem.SubItems.Add(TT.Hour);
    end;
    ListView.Items.EndUpdate;
    if BrowserCheckBox.Checked then
      SubmitButtonClick(Sender);
  end;
end;

procedure TSearchForm.SubmitButtonClick(Sender: TObject);
var ToPass : String;
begin
  ToPass := URLEdit.Text+'&Departure='+DepartureComboBox.Items[DepartureComboBox.ItemIndex];
  ToPass := ToPass+'&Arrival='+ArrivalComboBox.Items[ArrivalComboBox.ItemIndex];
  ToPass := ToPass+'&Hour='+Copy(HourEdit.Text,1,2);
  ToPass := ToPass+'&Minute='+Copy(HourEdit.Text,4,2);
  ToPass := ToPass+'&WeekDay='+ComboBox.Items[ComboBox.ItemIndex];
  Shellexecute(0,nil,Pchar(ToPass),nil,nil,SW_NORMAL);
end;

procedure TSearchForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  HourEdit.Text := '';
  RoutesDataModule.URLLocation := URLEdit.Text;
end;

procedure TSearchForm.FormShow(Sender: TObject);
begin

  URLEdit.Text := RoutesDataModule.URLLocation;
  with RoutesDataModule do begin
    CreateRouteStructure;
    TownListQuery.Close;
    TownListQuery.Open;
    TownListQuery.First;
    DepartureComboBox.Items.Clear;
    ArrivalComboBox.Items.Clear;
    while not TownListQuery.Eof do begin
      DepartureComboBox.Items.Add(TownListQuery.Fields[0].AsString);
      ArrivalComboBox.Items.Add(TownListQuery.Fields[0].AsString);
      TownListQuery.Next;
    end;
  end;

end;

procedure TSearchForm.ListViewCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
var Time1, Time2 : String;
begin
  Time1 := Item1.SubItems.Strings[1];
  Time2 := Item2.SubItems.Strings[1];
  Compare := CompareStr(Time1,Time2);
end;

procedure TSearchForm.ExitButtonClick(Sender: TObject);
begin
  Close
  
end;

end.
