{*****************************************************************************
 *
 *  uPageProducer.pas - Page producer to create HTML answer to query
 *
 *  Copyright (c) 1999/2000 Diego Amicabile
 *
 *  Author:     Diego Amicabile
 *  E-mail:     diegoami@yahoo.it
 *  Homepage:   http://www.geocities.com/diegoami
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation;
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *----------------------------------------------------------------------------
 *
 *  Revision history:
 *
 *     DATE     REV                 DESCRIPTION
 *  ----------- --- ----------------------------------------------------------
 *
 *****************************************************************************}

unit uPageProducer;

interface

uses
  Windows, Messages, SysUtils, Classes, HTTPApp, Db, DBTables, uSearchRoutine, DataMod, ObjGraph;

type

  TWebModule1 = class(TWebModule)
    PageProducer1: TPageProducer;
    RouteSearchQuery: TQuery;
    HourQuery: TQuery;
    TownsQuery: TQuery;
    procedure WebModule1WebActionItem1Action(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure PageProducer1HTMLTag(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings;
      var ReplaceText: String);
    procedure WebModule1Create(Sender: TObject);
    procedure WebModule1Destroy(Sender: TObject);
  private
    ConnectionList  : TList;
    AllConnections  : TStringList;
    Arrival      : String;
    Departure    : String;
    Hour         : String;
    WeekDay      : String;
    DatabaseName : String;
    HourString   : String;
    MinuteString : String;
    F            : TextFile;
    function ConnectionToString(EdgeList : TEdgeList) : String;
  public
    { Public declarations }
  end;

var
  WebModule1: TWebModule1;

  ScriptDir               :String;
implementation

{$R *.DFM}


procedure TWebModule1.WebModule1WebActionItem1Action(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);

  begin

    Arrival := Request.QueryFields.Values['Arrival'];
    Departure := Request.QueryFields.Values['Departure'];

    HourString := Request.QueryFields.Values['Hour'];
    if length(HourString) = 1 then
             HourString := '0'+HourString;
    MinuteString := Request.QueryFields.Values['Minute'];
    if length(MinuteString) = 1 then
             MinuteString := '0'+MinuteString;
    Hour := HourString + '.'+ MinuteString;

    WeekDay := Request.QueryFields.Values['WeekDay'];
    DatabaseName := Request.QueryFields.Values['Database'];

    if DatabaseName = '' then
       DatabaseName := 'data';
    if ValidHour(Hour) then
      Response.Content := PageProducer1.Content
    else
      Response.Content := 'Invalid time';
end;

procedure TWebModule1.PageProducer1HTMLTag(Sender: TObject; Tag: TTag;
  const TagString: String; TagParams: TStrings; var ReplaceText: String);

  var EdgeList : TEdgeList;
    Day : Char;
  begin
    if CompareText(TagString, 'Departure') = 0 then
      ReplaceText := Departure
    else if CompareText(TagString, 'Destination') = 0 then
      ReplaceText := Arrival
    else if CompareText(TagString, 'Hour') = 0 then
      ReplaceText := Hour
    else if CompareText(TagString, 'Day') = 0 then
      ReplaceText := WeekDay
    else if CompareText(TagString, 'Params') = 0 then begin
      if Pos('\',DatabaseName) > 0 then
         ReplaceText := 'Security Violation'
      else begin
         RoutesDataModule := TRoutesDataModule.Create(self);
         RoutesDataModule.Update(ScriptDir+'\'+DatabaseName);
         RoutesDataModule.CreateRouteStructure;
         Writeln(F,'Departure='+Departure);
         Writeln(F,'Arrival='+Arrival);
         Writeln(F,'Hour='+Hour);
         Writeln(F,'Day='+DayToChar(WeekDay));
         EdgeList := RoutesDataModule.GetConnectionList(Departure, Arrival, Hour, DayToChar(WeekDay));
         ReplaceText := ConnectionToString(EdgeList);
         Write(F,ReplaceText);
       end;
    end;
end;

function TWebModule1.ConnectionToString(EdgeList : TEdgeList) : String;
var ResultString : String;
    j : integer;
    Edge : TRouteEdge;
    ConnString : String;
    TownFrom, TownTo : TTownNode;
begin
  ConnString := '';
  ConnString := ConnString+'<BR>';
  ConnString := ConnString+'<TABLE BORDER=2 RULES=ALL> <TR><TD>Departure From</TD><TD>At</TD><TD>Destination</TD><TD>At</TD><TD>Vehicle</TD></TR>';
  for j := 0 to EdgeList.Count-1 do begin
    Edge := TRouteEdge(EdgeList.Items[j]);
    TownFrom := Edge.FromNode As TTownNode;
    TownTo := Edge.ToNode As TTownNode;
    ConnString := ConnString+'<TR><TD>'+TownFrom.Town.Str+'</TD>';
    ConnString := ConnString+'<TD>'+Edge.DepHour+'</TD>';
    ConnString := ConnString+'<TD>'+TownTo.Town.Str+'</TD>';
    ConnString := ConnString+'<TD>'+TownTo.Hour+'</TD>';
    ConnString := ConnString+'<TD>'+Edge.Vehicle+'</TD></TR>';
  end;
  ConnString := ConnString+'</TABLE><BR>';
  result := ConnString;
end;

procedure TWebModule1.WebModule1Create(Sender: TObject);
var
  FN: array[0..MAX_PATH- 1] of char;
  ScriptName    : String;
begin

  SetString(ScriptName, FN, GetModuleFileName(hInstance, FN, SizeOf(FN)));
  ScriptDir    := ExtractFileDir(ScriptName);
  AssignFile(F, 'logger.txt');
  Rewrite(F);
end;

procedure TWebModule1.WebModule1Destroy(Sender: TObject);
begin
  Close(F);
end;

end.
